@@ -16,11 +16,11 @@
 
 # pylint: disable-msg=C0301
 
-prefix = '/usr/local'
-data_path = '/usr/local/share/sugar/data'
-shell_path = '/usr/local/share/sugar/shell'
-locale_path = '/usr/local/share/locale'
-ext_path = '/usr/local/share/sugar/extensions'
-activities_path = "/usr/local/share/sugar/activities"
-version = '0.84.16'
+prefix = '/home/erikos/sugar-jhbuild/install'
+data_path = '/home/erikos/sugar-jhbuild/install/share/sugar/data'
+shell_path = '/home/erikos/sugar-jhbuild/install/share/sugar/shell'
+locale_path = '/home/erikos/sugar-jhbuild/install/share/locale'
+ext_path = '/home/erikos/sugar-jhbuild/install/share/sugar/extensions'
+activities_path = "/home/erikos/sugar-jhbuild/install/share/sugar/activities"
+version = '0.84.30'
 
